
/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file board_easy50712.c
   \date 2006-10-01
   \brief Interface implementation for DANUBE board.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "board_easy334.h"


/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/** How channels are represented by device, socket.
    Device /dev/vin11 will get status, data from phone channel 0 and
    data channel 0. If at startup phone channel 0 is mapped to data channel 1
    then, phone will send actions to dev/vin11 and data will send actions to
    /dev/vin12.
    -1 means no channel at startup */

/** Max phones on VIP (analog channels) */
enum { MAX_PHONES_VIP = 4 };

/** Max data channels on VIP */
enum { MAX_CODER_CH_VIP = 4 };

/** Max pcm channels on VIP */
enum { MAX_PCM_CH_VIP = 8 };

#ifdef STREAM_1_1

/** Name of board */
static const char* BOARD_NAME_EASY334 = "EASY332";

static const STARTUP_MAP_TABLE_t STARTUP_MAPPING_VIP[MAX_PHONES_VIP] =
{
   /** phone, data, pcm */
   {  0,  0,  0  },
   {  1,  1,  1  },
   {  2,  2,  2  },
   {  3,  3,  3  }
   /* others are free and can be used later for conferencing */
};

#endif /* STREAM_1_1 */


/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t BOARD_Easy334_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_return_t BOARD_Easy334_PrepareNames(BOARD_t* pBoard);
IFX_return_t BOARD_Easy334_Set_CPLD_Reg_For_PCM(IFX_boolean_t fMaster,
                                                BOARD_t* pBoard,
                                                IFX_boolean_t fLocalLoop);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Prepare board structure (reserve memory, inicialize, ...).

   \param pBoard - pointer to Board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called after system inicialization.
*/
IFX_return_t BOARD_Easy334_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_int32_t i = 0;
   IFX_return_t ret;

   /* check input arguments */
   if (pBoard == IFX_NULL || pPath == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* First three variables (id, type, sub type) set before this func is called */

   /* File descriptors not open */
   pBoard->nSystem_FD = -1;
   pBoard->nDevCtrl_FD = -1;
   /* Set flag to use system FD */
   pBoard->nUseSys_FD = IFX_TRUE;

   pBoard->pszBoardName = BOARD_NAME_EASY334;

   if (BOARD_Easy334_PrepareNames(pBoard) == IFX_ERROR)
      return IFX_ERROR;

   /** Startup mapping table */
   pBoard->pChStartupMapping = STARTUP_MAPPING_VIP;

#ifdef USE_FILESYSTEM
   /** FW files */
   pBoard->pszDRAM_FW_File = sDRAMFile_VIP;
   pBoard->pszPRAM_FW_File[0] = sPRAMFile_VIP;
   pBoard->pszPRAM_FW_File[1] = sPRAMFile_VIP_Old;
   pBoard->pszBBD_CRAM_File = sBBD_CRAM_File_VIP;
#ifndef VXWORKS
   /* if using default download path and .tapidemorc file exists
      then change downloads path to specified in .tapidemorc */
   ret = Common_CheckDownloadPath(pPath, pBoard, pCPUDevice);
   if (IFX_SUCCESS != ret) 
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
            ("Using default download path: %s.\n", 
             DEF_DOWNLOAD_PATH));
   }
#endif /* VXWORKS */
#endif /* USE_FILESYSTEM */   

   /** Pointer to device dependent structure */
   pBoard->pDevDependStuff = (IFX_void_t *) &VineticDevInitStruct;

   pBoard->nMaxAnalogCh = MAX_PHONES_VIP;
   pBoard->nMaxCoderCh = MAX_CODER_CH_VIP;
   pBoard->nMaxPCM_Ch = MAX_PCM_CH_VIP

   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->nUsedChannels = pBoard->nMaxCoderCh;
   }
   else if (pBoard->nMaxAnalogCh > 0)
   {
      /* use max analog ch, which must be != 0, if also 0 then ERROR */
      pBoard->nUsedChannels = pBoard->nMaxAnalogCh;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Analog channels are missing! (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->pDataChStat = calloc(pBoard->nMaxCoderCh, sizeof(VOIP_DATA_CH_t));
      /* \todo maybe check for max ch number available and use this number */
      pBoard->nCh_FD = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
      pBoard->rgnSockets = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
   }

   pBoard->rgoPhones = calloc(pBoard->nMaxAnalogCh, sizeof(PHONE_t));

   if ((pBoard->pDataChStat == IFX_NULL) || (pBoard->rgoPhones == IFX_NULL)
       || (pBoard->nCh_FD == IFX_NULL) || (pBoard->rgnSockets == IFX_NULL))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR Allocate memory to prepare board. "
             "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   for (i = 0; i < pBoard->nMaxCoderCh; i++)
   {
       pBoard->nCh_FD[i] = -1;
       pBoard->rgnSockets[i] = -1;
   }

   /* Prepare array of free PCM channels  */
   pBoard->fPCM_ChFree = calloc(pBoard->nMaxPCM_Ch, sizeof(IFX_boolean_t));

   if (pBoard->fPCM_ChFree == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR allocate memory for PCM channels. "
             "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   /* All PCM channels are free at start */
   for (i = 0; i < pBoard->nMaxPCM_Ch; i++)
   {
       pBoard->fPCM_ChFree[i] = IFX_TRUE;
   }

   if (IFX_SUCCESS != Common_Set_FDs(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error setting fds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_return_t BOARD_Easy334_PrepareNames(BOARD_t* pBoard)
{
   IFX_char_t tmp_buf[10] = {0};

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /** Device name without numbers, like /dev/vmmc, /dev/vin, ... */
   strncpy(pBoard->pszChDevName, sCH_DEV_NAME_VIP, strlen(sCH_DEV_NAME));

   /** Device control name numbers, like /dev/vmmc[board id]0, ... */
   strncpy(pBoard->pszCtrlDevName, sCTRL_DEV_NAME_VIP, strlen(sCTRL_DEV_NAME));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID);
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);
   /*snprintf(tmp_buf, 2, "0");*/
   sprintf(tmp_buf, "0");
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);

   /** System device name, like /dev/easy334/[board id], ... */
   strncpy(pBoard->pszSystemDevName, sSYS_DEV_NAME_VIP,
           strlen(sSYS_DEV_NAME_VIP));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID - 1);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID - 1);
   strncat(pBoard->pszSystemDevName, tmp_buf, 1);

   return IFX_SUCCESS;
}

/**
   Configure CPLD register for PCM initialization.

   \param fMaster - flag indicating if this board will be master (1) or
                    slave (0)
   \param pBoard - pointer to board
   \param fLocalLoop - 1 if PCM local loop is used, otherwise nothing (0).

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called after system inicialization.
*/
IFX_return_t BOARD_Easy3332_Set_CPLD_Reg_For_PCM(IFX_boolean_t fMaster,
                                                 BOARD_t* pBoard,
                                                 IFX_boolean_t fLocalLoop)
{
   IFX_uint8_t reg1_value = 0;
   IFX_uint8_t reg2_value = 0;
   IFX_uint8_t reg3_value = 0;
   EASY334_Reg_t reg_vip;   
   EASY3332_Reg_t reg_cpe;
   IFX_return_t ret = IFX_SUCCESS;


   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Read register */
   memset(&reg_vip, 0, sizeof(EASY334_Reg_t));
   
   reg_vip.nRegOffset = CMDREG1_OFF_VIP;
   /* Read register */
   ret = ioctl(pBoard->nSystem_FD, FIO_EASY334_CPLDREG_READ,
               (IFX_int32_t) &reg_vip);
   if (ret == IFX_SUCCESS)
   {
      reg1_value = reg_vip.nRegVal;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err reading CPLD register. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&reg_vip, 0, sizeof(EASY334_Reg_t));
   
   reg_vip.nRegOffset = CMDREG2_OFF_VIP;
   /* Read register */
   ret = ioctl(pBoard->nSystem_FD, FIO_EASY334_CPLDREG_READ,
               (IFX_int32_t) &reg_vip);
   if (ret == IFX_SUCCESS)
   {
      reg1_value = reg_vip.nRegVal;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err reading CPLD register. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   
   memset(&reg_vip, 0, sizeof(EASY334_Reg_t));
   
   reg_vip.nRegOffset = CMDREG3_OFF_VIP;
   /* Read register */
   ret = ioctl(pBoard->nSystem_FD, FIO_EASY334_CPLDREG_READ,
               (IFX_int32_t) &reg_vip);
   if (ret == IFX_SUCCESS)
   {
      reg1_value = reg_vip.nRegVal;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err reading CPLD register. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Read CMDREG%d 0x%x\n", 1, reg1_value));
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Read CMDREG%d 0x%x\n", 2, reg2_value));
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Read CMDREG%d 0x%x\n", 3, reg3_value));

   /* Set some common values for both MASTER and SLAVE board */
   reg1_value = 0;
   reg1_value |= VRES_VIP;
   reg2_value = 0;
   reg2_value |= HIGHWAY_B;
   reg2_value |= MICRO_CONTROLER_MPC_VIN;
   reg2_value |= MOTOR_16_BIT_AND_PCM;
   reg3_value = 0;
   reg3_value |= MCLK_IS_DCL;
   reg3_value |= IOM_2_CLK_OUT;
   reg3_value |= FREQ_2048_KHZ_8_KHZ;

   if (IFX_TRUE == fMaster)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
            ("Board mode set to PCM MASTER mode\n"));
      reg3_value |= MASTER_VIP;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
            ("Board mode set to PCM SLAVE mode\n"));
      reg3_value &= ~MASTER_VIP;
   }

   memset(&reg_vip, 0, sizeof(EASY334_Reg_t));
   
   reg_vip.nRegOffset = CMDREG1_OFF_VIP;
   reg_vip.nRegVal = reg1_value;
   /* Read register */
   ret = ioctl(pBoard->nSystem_FD, FIO_EASY334_CPLDREG_WRITE,
               (IFX_int32_t) &reg_vip);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err writing value 0x%0X to CPLD register. "
            "(File: %s, line: %d)\n", reg_vip.nRegVal, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&reg_vip, 0, sizeof(EASY334_Reg_t));
   
   reg_vip.nRegOffset = CMDREG2_OFF_VIP;
   reg_vip.nRegVal = reg2_value;
   /* Read register */
   ret = ioctl(pBoard->nSystem_FD, FIO_EASY334_CPLDREG_WRITE,
               (IFX_int32_t) &reg_vip);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err writing value 0x%0X to CPLD register. "
            "(File: %s, line: %d)\n", reg_vip.nRegVal, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&reg_vip, 0, sizeof(EASY334_Reg_t));
   
   reg_vip.nRegOffset = CMDREG3_OFF_VIP;
   reg_vip.nRegVal = reg3_value;
   /* Read register */
   ret = ioctl(pBoard->nSystem_FD, FIO_EASY334_CPLDREG_WRITE,
               (IFX_int32_t) &reg_vip);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err writing value 0x%0X to CPLD register. "
            "(File: %s, line: %d)\n", reg_vip.nRegVal, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Write CMDREG%d 0x%x\n", 1, reg1_value));
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Write CMDREG%d 0x%x\n", 2, reg2_value));
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Write CMDREG%d 0x%x\n", 3, reg3_value));

   return IFX_SUCCESS;
} /* BOARD_Easy3332_Set_CPLD_Reg_For_PCM() */

/* ============================= */
/* Global function definition    */
/* ============================= */
/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_void_t BOARD_Easy334_RemoveBoard(BOARD_t* pBoard)
{
   if (pBoard->pDataChStat != IFX_NULL)
   {
      free(pBoard->pDataChStat);
      pBoard->pDataChStat = IFX_NULL;
   }

   if (pBoard->rgoPhones != IFX_NULL)
   {
      free(pBoard->rgoPhones);
      pBoard->rgoPhones = IFX_NULL;
   }

   if (pBoard->rgoFXOs != IFX_NULL)
   {
      free(pBoard->rgoFXOs);
      pBoard->rgoFXOs = IFX_NULL;
   }

   if (pBoard->nCh_FD != IFX_NULL)
   {
      free(pBoard->nCh_FD);
      pBoard->nCh_FD = IFX_NULL;
   }

   if (pBoard->rgnSockets != IFX_NULL)
   {
      free(pBoard->rgnSockets);
      pBoard->rgnSockets = IFX_NULL;
   }

   if (pBoard->nFxoCh_FD != IFX_NULL)
   {
      free(pBoard->nFxoCh_FD);
      pBoard->nFxoCh_FD = IFX_NULL;
   }

   if (pBoard->fPCM_ChFree != IFX_NULL)
   {
      free(pBoard->fPCM_ChFree);
      pBoard->fPCM_ChFree = IFX_NULL;
   }
}

/**
   Read version after succesfull initialization and display it.

   \param pBoard - pointer to board
   \param pProgramArg - pointer to program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      This must be done only after a successfull basic device initialization
*/
IFX_return_t BOARD_Easy334_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg)
{
   IFX_int32_t phone_ch_cnt = 0;
   IFX_int32_t data_ch_cnt = 0;
   IFX_int32_t pcm_ch_cnt = 0;
   IFX_boolean_t we_are_master = IFX_FALSE;
   IFX_boolean_t use_pcm_local_loop = IFX_FALSE;

    /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy334_PrepareBoard(pBoard, pProgramArg->sPathToDwnldFiles)
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize chip */
   if (DEVICE_VINETIC_VIP_Init(pBoard, pProgramArg->sPathToDwnldFiles) 
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize PCM if used (just set values to some CPLD registers) */
   if ((pProgramArg->oArgFlags.nPCM_Master)
       || (pProgramArg->oArgFlags.nPCM_Slave))
   {
      if ((pBoard->nID - 1) == 0)
      {
         if (pProgramArg->oArgFlags.nPCM_Master)
         {
            we_are_master = IFX_TRUE;
         }

         if (pProgramArg->oArgFlags.nPCM_Loop)
         {
            use_pcm_local_loop = IFX_TRUE;
         }

         if (BOARD_Easy334_Set_CPLD_Reg_For_PCM(we_are_master, pBoard,
                                         use_pcm_local_loop) != IFX_SUCCESS)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("PCM initialization failed. (File: %s, line: %d)\n",
                  __FILE__, __LINE__));
            return IFX_ERROR;
         }
      }
   }


   /* Display number of all channels on this board. */
   Common_GetCapabilities(pBoard, &phone_ch_cnt, &data_ch_cnt, &pcm_ch_cnt);

   return IFX_SUCCESS;
}



