
/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file board_easy50510.c
   \date 2006-10-01
   \brief Interface implementation for EASY50510 board.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "board_easy50510.h"
#include "board_easy50712.h"
#include "common_fxo.h"


/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/** How channels are represented by device, socket.
    Device /dev/vin11 will get status, data from phone channel 0 and
    data channel 0. If at startup phone channel 0 is mapped to data channel 1
    then, phone will send actions to dev/vin11 and data will send actions to
    /dev/vin12.
    -1 means no channel at startup */

/** Max phones on CPE (analog channels)
    It is used only for STARTUP_MAPPING_CPE[] table. 
    Number of analog channels used in program is read from drivers 
    using IFX_TAPI_CAP_LIST */
enum { MAX_PHONES_CPE = 2 };

/** Max DUSLIC channels on CPE */
enum { MAX_DUSLIC_CH_CPE = 0 };

/** Number of PCM highways */
enum { PCM_HIGHWAYS = 1 };

/** Name of board */
static const char* BOARD_NAME_EASY50510 = "EASY50510";

/** Holds default phone channel to data, pcm channel mappings
    defined at startup. */
static const STARTUP_MAP_TABLE_t STARTUP_MAPPING_CPE[MAX_PHONES_CPE] =
{
   /** phone, data, pcm */
   {  0,  0,  0  },
   {  1,  1,  1  }
   /* others are free and can be used later for conferencing */
};


/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t BOARD_Easy50510_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_return_t BOARD_Easy50510_PrepareNames(BOARD_t* pBoard);
IFX_return_t BOARD_Easy50712_CfgPCMInterface(BOARD_t* pBoard,
                                             IFX_boolean_t hMaster);
IFX_return_t BOARD_Easy50510_InitSystem(BOARD_t* pBoard);
IFX_return_t BOARD_Easy50510_GetCapabilities(BOARD_t* pBoard,
                                             IFX_char_t* pPath,
                                             IFX_int32_t *pPhoneChCnt,
                                             IFX_int32_t *pDataChCnt,
                                             IFX_int32_t *pPCM_ChCnt);
IFX_void_t BOARD_Easy50510_RemoveBoard(BOARD_t* pBoard);
IFX_return_t BOARD_Easy50510_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Prepare board structure (reserve memory, inicialize, ...).

   \param pBoard - pointer to Board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t BOARD_Easy50510_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t* pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t* pCPUDevice = IFX_NULL;
#ifdef FXO
   TAPIDEMO_DEVICE_FXO_t* pFXODevice = IFX_NULL;
#endif /* FXO */
   IFX_return_t ret;

   /* check input arguments */
   if ((pBoard == IFX_NULL) || (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Variables id and type set before this func is called */

   /* File descriptors not open */
   pBoard->nSystem_FD = -1;
   pBoard->nDevCtrl_FD = -1;
   /* Set flag NOT to use system FD */
   pBoard->nUseSys_FD = IFX_FALSE;

   /* allocate memory for CPU device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for CPU Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Init structure for CPU device and add it to the board */
   pDevice->nType = TAPIDEMO_DEV_CPU;
   pCPUDevice = &pDevice->uDevice.stCpu;
   /** Pointer to device dependent structure */
   pCPUDevice->pDevDependStuff = (IFX_void_t *) &VineticDevInitStruct;
   pCPUDevice->AccessMode = ACCESS_MODE;
   pCPUDevice->nBaseAddress = BASE_ADDRESS;
   pCPUDevice->nIrqNum = IRQ_NUMBER;
   pCPUDevice->SetPin_Direction_onGPIO = BOARD_Easy50712_SetPin_Direction_onGPIO;
   pCPUDevice->Reset = BOARD_Easy50712_Reset;
#ifdef USE_FILESYSTEM
   /** FW files */
   pCPUDevice->pszDRAM_FW_File = sDRAMFile_CPE;
   pCPUDevice->pszPRAM_FW_File[0] = sPRAMFile_CPE;
   pCPUDevice->pszPRAM_FW_File[1] = sPRAMFile_CPE_Old;
   pCPUDevice->pszBBD_CRAM_File = sBBD_CRAM_File_CPE;
#ifndef VXWORKS
   /* if using default download path and .tapidemorc file exists
      then change downloads path to specified in .tapidemorc */
   ret = Common_CheckDownloadPath(pPath, pBoard, pCPUDevice);
   if (IFX_SUCCESS != ret) 
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
            ("Using default download path: %s.\n", 
             DEF_DOWNLOAD_PATH));
   }
#endif /* VXWORKS */
#endif /* USE_FILESYSTEM */

   if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
   {
      free(pDevice);
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not added to the board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* allocate memory for FXO device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for FXO Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
#ifdef FXO
   if (0 < pBoard->nMaxFxoCh)
   {
      /* Init structure for FXO device and add it to the board */
      pDevice->nType = TAPIDEMO_DEV_FXO;
      pFXODevice = &pDevice->uDevice.stFxo;
      pFXODevice->nFxoType = FXO_TYPE_NONE;
#ifdef DUSLI_FXO
      pFXODevice->nFxoType = FXO_TYPE_DUSLIC;
#endif
      pFXODevice->nIrqNum = INT_NUM_IM2_IRL31;
      pFXODevice->Reset = BOARD_Easy50712_Reset;
      pFXODevice->SetPin_Direction_onGPIO = BOARD_Easy50712_SetPin_Direction_onGPIO;
      if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
      {
         free(pDevice);
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("ERR, FXO device not added to the board. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
#endif /* FXO */
   if (DEVICE_VINETIC_CPE_Register(pCPUDevice) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not registered. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy50510_PrepareNames(pBoard) == IFX_ERROR)
      return IFX_ERROR;

   /** Startup mapping table */
   pBoard->pChStartupMapping = STARTUP_MAPPING_CPE;
   /* Set PCM configuration (master, ...) over GPIO port */

   /* karol - check if it is necessary. */
   if (BOARD_Easy50712_SetPCM_Master_overGPIO() != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error set PCM configuration over GPIO port (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /** Set pin direction to output */
   if (BOARD_Easy50712_SetPin_Direction_onGPIO(GPIO_VINETIC_EASY50510) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error set pin direction (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   pBoard->nMaxFxoCh = MAX_DUSLIC_CH_CPE;

   /* Basic device initialization */
   if (BOARD_Easy50510_InitSystem(pBoard) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   if (IFX_ERROR == BOARD_Easy50510_GetCapabilities(pBoard, pPath,
                                                    &pBoard->nMaxAnalogCh,
                                                    &pBoard->nMaxCoderCh,
                                                    &pBoard->nMaxPCM_Ch))
   {
      return IFX_ERROR;
   }

   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->nUsedChannels = pBoard->nMaxCoderCh;
   }
   else if (pBoard->nMaxAnalogCh > 0)
   {
      /* use max analog ch, which must be != 0, if also 0 then ERROR */
      pBoard->nUsedChannels = pBoard->nMaxAnalogCh;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Analog channels are missing! (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->pDataChStat = calloc(pBoard->nMaxCoderCh, sizeof(VOIP_DATA_CH_t));
      /* \todo maybe check for max ch number available and use this number */
      pBoard->nCh_FD = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
      pBoard->rgnSockets = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
   }

   pBoard->rgoPhones = calloc(pBoard->nMaxAnalogCh, sizeof(PHONE_t));

   if ((pBoard->pDataChStat == IFX_NULL) || (pBoard->rgoPhones == IFX_NULL) ||
       (pBoard->nCh_FD == IFX_NULL) || (pBoard->rgnSockets == IFX_NULL))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR Allocate memory to prepare board. "
             "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }
   /* check if any FXO channel is available */
   if (0 < pBoard->nMaxFxoCh)
   {
      pBoard->rgoFXOs = calloc(pBoard->nMaxFxoCh, sizeof(FXO_t));
      if (pBoard->rgoFXOs == IFX_NULL)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Err, Allocate memory for FXO on board failed."
               " (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   } /* if (0 < pBoard->nMaxFxoCh) */

   for (i = 0; i < pBoard->nMaxCoderCh; i++)
   {
       pBoard->nCh_FD[i] = -1;
       pBoard->rgnSockets[i] = -1;
   }
   /* check if any FXO channel is available */
   if (0 < pBoard->nMaxFxoCh)
   {
      /* DUSLIC */
      pBoard->nFxoCh_FD = calloc(pBoard->nMaxFxoCh, sizeof(IFX_int32_t));
   
      if (pBoard->nFxoCh_FD == IFX_NULL)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("ERR allocate memory for duslic channels. "
                "(File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;         
      }
   
      for (i = 0; i < pBoard->nMaxFxoCh; i++)
      {
          pBoard->nFxoCh_FD[i] = -1;
      }
   }

   /* Prepare array of free PCM channels  */
   pBoard->fPCM_ChFree = calloc(pBoard->nMaxPCM_Ch, sizeof(IFX_boolean_t));

   if (pBoard->fPCM_ChFree == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR allocate memory for PCM channels. "
             "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   /* All PCM channels are free at start */
   for (i = 0; i < pBoard->nMaxPCM_Ch; i++)
   {
       pBoard->fPCM_ChFree[i] = IFX_TRUE;
   }

   if (IFX_SUCCESS != Common_Set_FDs(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error setting fds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_return_t BOARD_Easy50510_PrepareNames(BOARD_t* pBoard)
{
   IFX_char_t tmp_buf[10] = {0};

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /** Device name without numbers, like /dev/vmmc, /dev/vin, ... */
   strncpy(pBoard->pszChDevName, sCH_DEV_NAME_CPE, strlen(sCH_DEV_NAME_CPE));

   /** Device control name numbers, like /dev/vmmc[board id]0, ... */
   strncpy(pBoard->pszCtrlDevName, sCTRL_DEV_NAME_CPE, 
           strlen(sCTRL_DEV_NAME_CPE));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID);
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);
   /*snprintf(tmp_buf, 2, "0");*/
   sprintf(tmp_buf, "0");
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);
#ifdef DUSLIC_FXO
   /** Duslic device name without numbers, like /dev/dus */
   strncpy(pBoard->pszFxoChDevName, sDUSLIC_CH_DEV_NAME, 
           strlen(sDUSLIC_CH_DEV_NAME));

   /** Device control name numbers, like /dev/dus[board id]0, ... */
   strncpy(pBoard->pszFxoCtrlDevName, sDUSLIC_CTRL_DEV_NAME, 
           strlen(sDUSLIC_CTRL_DEV_NAME));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID);
   strncat(pBoard->pszFxoCtrlDevName, tmp_buf, 1);
   /*snprintf(tmp_buf, 2, "0");*/
   sprintf(tmp_buf, "0");
   strncat(pBoard->pszFxoCtrlDevName, tmp_buf, 1);
#endif /* DUSLIC_FXO */
   return IFX_SUCCESS;
}

#ifndef STREAM_1_1
/**
   Configure PCM interface, can define clock, master/slave mode, ...

   \param pBoard  - pointer to board
   \param hMaster  - IFX_TRUE - master mode, IFX_FALSE - slave mode

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
   
   \remark 
*/
IFX_return_t BOARD_Easy50510_CfgPCMInterface(BOARD_t* pBoard, 
                                             IFX_boolean_t hMaster)
{

   IFX_int32_t j;
   IFX_TAPI_PCM_IF_CFG_t pcm_if;
   IFX_return_t ret;

   if (IFX_NULL == pBoard)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_FALSE;
   }

   memset(&pcm_if, 0, sizeof(IFX_TAPI_PCM_IF_CFG_t));

   if (hMaster)
   {
      pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_MASTER;
   }
   else
   {
      pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_SLAVE_AUTOFREQ;
   }

   pcm_if.nDCLFreq = IFX_TAPI_PCM_IF_DCLFREQ_2048;
   pcm_if.nDoubleClk = IFX_DISABLE;

   pcm_if.nSlopeTX = IFX_TAPI_PCM_IF_SLOPE_RISE;
   pcm_if.nSlopeRX = IFX_TAPI_PCM_IF_SLOPE_FALL;
   pcm_if.nOffsetTX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nOffsetRX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nDrive = IFX_TAPI_PCM_IF_DRIVE_ENTIRE;
   pcm_if.nShift = IFX_DISABLE;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
         ("%s: Configure PCM interface as %s, %s kHz\n",
          pBoard->pszBoardName,
          (hMaster ? "master":"slave"), TAPIDEMO_PCM_FREQ_STR[pcm_if.nDCLFreq]));

   for (j = 0; j < PCM_HIGHWAYS; j++)
   {
      ret = ioctl(pBoard->nDevCtrl_FD, IFX_TAPI_PCM_IF_CFG_SET,
                  (IFX_int32_t) &pcm_if);
      if (ret != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
               ("Err configure PCM interface (File: %s, line: %d)\n", 
                __FILE__, __LINE__));
         return IFX_ERROR;
      }
   } /* for */

   /* set PCM frequency - should be only set for main board - not extension */
   /* oPCM_Data.nPCM_Rate = pcm_if.nDCLFreq; */

   return IFX_SUCCESS;
} /* BOARD_Easy50510_CfgPCMInterface() */

#else /* !STREAM_1_1 */

IFX_return_t BOARD_Easy50510_CfgPCMInterface(BOARD_t* pBoard, 
                                             IFX_boolean_t hMaster)
{
   return IFX_SUCCESS;
} /* BOARD_Easy50510_CfgPCMInterface() */
#endif /* !STREAM_1_1 */

/**
   Get capabilities, basically number of phone, data and pcm channels.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download
   \param pPhoneChCnt - pointer to phone channels count
   \param pDataChCnt  - pointer to data channels count
   \param pPCM_ChCnt  - pointer to pcm channels count

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy50510_GetCapabilities(BOARD_t* pBoard,
                                             IFX_char_t* pPath,
                                             IFX_int32_t *pPhoneChCnt,
                                             IFX_int32_t *pDataChCnt,
                                             IFX_int32_t *pPCM_ChCnt)
{
   IFX_return_t ret = IFX_ERROR;

   if ((IFX_NULL == pPhoneChCnt) || (IFX_NULL == pDataChCnt) ||
       (IFX_NULL == pPCM_ChCnt) || (pBoard == IFX_NULL) ||
       (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   ret = DEVICE_VINETIC_CPE_setupChannel(pBoard, pPath);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Channel 0 is not initialized. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      close(pBoard->nDevCtrl_FD);
      return IFX_ERROR;
   }

   ret = Common_GetCapabilities(pBoard, pPhoneChCnt, pDataChCnt, pPCM_ChCnt);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Capabilities not read. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
   }

   if (*pPhoneChCnt > MAX_PHONES_CPE)
   {
      /* If MAX_PHONES_CPE is too small than STARTUP_MAPPING_CPE[] table 
         has too small size and TAPIDEMO will from wrong place */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, MAX_PHONES_CPE is too small. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return ret;
}

/**
   Basic initialization prior to vinetic initialization.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      - In case of 8bit SPI, for the CPE system, base address  should be set to
        0x1F if not otherwise programmed by bootstrapping
      - Caller must make sure that the access mode is  supported
*/
IFX_return_t BOARD_Easy50510_InitSystem(BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;

   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Store chip count */
   pBoard->nChipCnt = 1;

   if (0 >= pBoard->nDevCtrl_FD)
   {
      pBoard->nDevCtrl_FD = Common_Open(pBoard->pszCtrlDevName);
      if (0 >= pBoard->nDevCtrl_FD)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Invalid device control file descriptor %s.\n"
               "(File: %s, line: %d)\n",
               pBoard->pszCtrlDevName, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
   
   /* activate chip reset */
   ret = BOARD_Easy50712_Reset(pBoard, (IFX_int32_t)GPIO_VINETIC_EASY50510, 
                               IFX_TRUE);
   if (ret == IFX_SUCCESS) 
   {
      ret = DEVICE_VINETIC_CPE_BasicInit(pBoard);
   }
      
   if (ret == IFX_SUCCESS) 
   {
      /* clear reset to allow vinetic access */
      ret = BOARD_Easy50712_Reset(pBoard, (IFX_int32_t)GPIO_VINETIC_EASY50510, 
                                  IFX_FALSE);
   }

   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERROR: Basic Device Initialization fails"
            "(File: %s, line: %d)\n",
             __FILE__, __LINE__));
   }

   return IFX_SUCCESS;
} /* BVINETIC_InitSystem() */

/* ============================= */
/* Global function definition    */
/* ============================= */
/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_void_t BOARD_Easy50510_RemoveBoard(BOARD_t* pBoard)
{
   if (pBoard->pDataChStat != IFX_NULL)
   {
      free(pBoard->pDataChStat);
      pBoard->pDataChStat = IFX_NULL;
   }

   if (pBoard->rgoPhones != IFX_NULL)
   {
      free(pBoard->rgoPhones);
      pBoard->rgoPhones = IFX_NULL;
   }

   if (pBoard->rgoFXOs != IFX_NULL)
   {
      free(pBoard->rgoFXOs);
      pBoard->rgoFXOs = IFX_NULL;
   }

   if (pBoard->nCh_FD != IFX_NULL)
   {
      free(pBoard->nCh_FD);
      pBoard->nCh_FD = IFX_NULL;
   }

   if (pBoard->rgnSockets != IFX_NULL)
   {
      free(pBoard->rgnSockets);
      pBoard->rgnSockets = IFX_NULL;
   }

   if (pBoard->nFxoCh_FD != IFX_NULL)
   {
      free(pBoard->nFxoCh_FD);
      pBoard->nFxoCh_FD = IFX_NULL;
   }

   if (pBoard->fPCM_ChFree != IFX_NULL)
   {
      free(pBoard->fPCM_ChFree);
      pBoard->fPCM_ChFree = IFX_NULL;
   }

   Common_RemoveDeviceList(pBoard);
}

/**
   Read version after succesfull initialization and display it.

   \param pBoard - pointer to board
   \param pProgramArg - pointer to program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      This must be done only after a successfull basic device initialization
*/
IFX_return_t BOARD_Easy50510_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg)
{
    /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy50510_PrepareBoard(pBoard, pProgramArg->sPathToDwnldFiles)
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize chip */
   if (DEVICE_VINETIC_CPE_Init(pBoard, pProgramArg->sPathToDwnldFiles) 
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize the PCM Interface. This is extension board
      so set PCM slave mode. */
   if (BOARD_Easy50510_CfgPCMInterface(pBoard, IFX_FALSE) == IFX_ERROR)
   {
       return IFX_ERROR;
   }
#ifdef FXO
   /* FXO */
   if (pProgramArg->oArgFlags.nFXO)
   {
      /* check if any FXO channel is available */
      if (0 < pBoard->nMaxFxoCh)
      {
#ifdef DUSLIC_FXO
         if (IFX_SUCCESS != Common_Set_Fxo_FDs(pBoard))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Error setting DUSLIC fds. (File: %s, line: %d)\n",
                  __FILE__, __LINE__));
            Common_Close_Fxo_FDs(pBoard);
            return IFX_ERROR;
         }
   
         if (IFX_SUCCESS != FXO_Init(pBoard, GPIO_DUSLIC_EASY50510,
                                     FXO_TYPE_DUSLIC))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("Error FXO initialization. (File: %s, line: %d)\n",
                  __FILE__, __LINE__));
            Common_Close_Fxo_FDs(pBoard);
            return IFX_ERROR;
         }
#endif /* DUSLIC_FXO */
      } /* if (0 < pBoard->nMaxFxoCh) */
   }
#endif /* FXO */
   return IFX_SUCCESS;
}

/**
   Register board.

   \param pBoard - pointer to board
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t BOARD_Easy50510_Register(BOARD_t* pBoard)
{
   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pBoard->pszBoardName = BOARD_NAME_EASY50510;
   pBoard->Init = BOARD_Easy50510_Init;
   pBoard->RemoveBoard = BOARD_Easy50510_RemoveBoard;
   return IFX_SUCCESS;
}



