
/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/
 
/**
   \file common_fxo.c
   \date 2006-11-28
   \brief This file contains implementation of FXO.

   It initializes FXO (FW, channels, ...) and uses PCM or channel mapping
   and timeslot usage. Basically when connection to FXO is started phone
   channel must be mapped to pcm channel and then pcm connection between this
   pcm channels and pcm channel on FXO is started.
*/
#ifdef LINUX
#include "tapidemo_config.h"
#endif

#include "ifx_types.h"
#include "tapidemo.h"

#ifdef FXO

/* ============================= */
/* Includes                      */
/* ============================= */
#ifdef DUSLIC_FXO
   #include "duslic_fxo.h"
#endif /* DUSLIC_FXO */

#ifdef TERIDIAN_FXO
   #include "teridian_fxo.h"
#endif /* TERIDIAN_FXO */

#ifdef HAVE_DRV_TAPI_HEADERS
    #include "drv_tapi_io.h"
#else
    #error "drv_tapi is missing"
#endif
#include "cid.h"
#include "pcm.h"
#include "voip.h"
#include "state_trans.h"


/* ============================= */
/* Global Structures             */
/* ============================= */

/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */

IFX_return_t FXO_ReleaseDataCh(FXO_t* pFXO, BOARD_t* pBoard);
IFX_return_t FXO_Start_CPTD(IFX_int32_t nDataCh_FD, IFX_int32_t nTone);
IFX_return_t FXO_Stop_CPTD(IFX_int32_t nDataCh_FD);

/* ============================= */
/* Local function definition     */
/* ============================= */

/**
   Unmap and release used data channel.
   
   \param pFXO - pointer to fxo
   \param pBoard - pointer to board
   
   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t FXO_ReleaseDataCh(FXO_t* pFXO, BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;

   /* check input parameters */
   TD_PTR_CHECK(pFXO, IFX_ERROR);
   TD_PTR_CHECK(pBoard, IFX_ERROR);
   TD_PARAMETER_CHECK((pFXO->nDataCh == NO_FREE_DATA_CH),
                      pFXO->nDataCh, IFX_ERROR);

   /* Unmap data channel from pcm channel */
   ret = PCM_MapToData(pFXO->nPCM_Ch, pFXO->nDataCh,
                       IFX_FALSE, pBoard);
   if (ret != IFX_SUCCESS)
   {
      /* error, could not unmap data ch from pcm ch */
      return ret;
   }

   if (VOIP_FreeDataCh(pFXO->nDataCh, pBoard) != IFX_SUCCESS)
   {
      /* ERR could not release reserved data ch */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("FXO No %d: %s (ch %d): "
            "Err, Could not release data channel %d, %s"
            "(File: %s, line: %d)\n", 
            (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
            (int) pFXO->nFxoCh, 
            pFXO->nDataCh, pBoard->pszBoardName,
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("FXO No %d: %s (ch %d): "
          "Free data ch: %d, %s\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh,
          (int) pFXO->nDataCh, pBoard->pszBoardName));
   
   pFXO->nDataCh = NO_FREE_DATA_CH;

   return ret;
}

/**
   Get FXO structure connected to data channel.

   \param nDataCh - number of data chennel
   \param pCtrl   - pointer to control structure
   \param pBoard  - pointer to board
   
   \return pointer to FXO_t structrue if connected to data channel or 
      IFX_NULL if structure not found
*/
FXO_t* FXO_GetFxoOfDataCh(IFX_int32_t nDataCh, CTRL_STATUS_t* pCtrl,
                          BOARD_t* pBoard)
{
   IFX_int32_t nFxoCh;
   IFX_int32_t i;

   /* check input parameters */
   TD_PARAMETER_CHECK((0 > nDataCh), nDataCh, IFX_NULL);
   TD_PTR_CHECK(pCtrl, IFX_NULL);
   TD_PTR_CHECK(pBoard, IFX_NULL);
   
   /* check if this is the extension board which has only FXO channels.
      Do not check this data channels becuase data channels from main board
      are used. */
   if ((pBoard->nBoard_IDX > 0) && (pBoard->nMaxAnalogCh == 0))
   {
      return IFX_NULL;
   }
   /* for all FXO channels on board */
   for (nFxoCh = 0; nFxoCh < pBoard->nMaxFxoCh; nFxoCh++)
   {
      /* check if structure allocated
         and if data channel number is same as searched */
      if ((IFX_NULL != pBoard->rgoFXOs) &&
          (pBoard->rgoFXOs[nFxoCh].nDataCh == nDataCh))
      {
         /* check if data channel connected to FXO */
         return &pBoard->rgoFXOs[nFxoCh];
      }
   } /* for all FXO channels on board */
   /* check if main(first) board */
   if (pBoard->nBoard_IDX == 0)
   {
      /* This is the main board. Chceck if there is connected extension board
         which has only FXO channels. In such situation extension board
         uses data channels from main board. */
      /* check all extension boards */
      for (i=1; i < pCtrl->nBoardCnt; i++)
      {
         /* if more than 0 FXO channels and no analog channels */
         if ((pCtrl->rgoBoards[i].nMaxFxoCh > 0) &&
             (pCtrl->rgoBoards[i].nMaxAnalogCh == 0))
         {
            /* for all FXO channels on extension board */
            for (nFxoCh = 0; nFxoCh < pCtrl->rgoBoards[i].nMaxFxoCh; nFxoCh++)
            {
               /* check if data channel connected to FXO */
               if ((IFX_NULL != pCtrl->rgoBoards[i].rgoFXOs) &&
                   (pCtrl->rgoBoards[i].rgoFXOs[nFxoCh].nDataCh == nDataCh))
               {
                  /* data channel is used by the extension board */
                  return &pCtrl->rgoBoards[i].rgoFXOs[nFxoCh];
               }
            } /* for all FXO channels on extension board */
         } /* if more than 0 FXO channels and no analog channels */
      } /* check all extension boards */
   } /* if main board */
   return IFX_NULL;
}

/**
   Start Call Progress Tone Detection
   
   \param nDataCh_FD - data channel file descriptor
   \param nTone - the index of the tone to detect

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t FXO_Start_CPTD(IFX_int32_t nDataCh_FD, IFX_int32_t nTone)
{
    IFX_return_t ret = IFX_SUCCESS;

   /* check input parameters */
   TD_PARAMETER_CHECK((nTone < 0), nTone, IFX_ERROR);
   TD_PARAMETER_CHECK((0 >= nDataCh_FD), nDataCh_FD, IFX_ERROR);

    IFX_TAPI_TONE_CPTD_t cpt;

    memset(&cpt, 0, sizeof(IFX_TAPI_TONE_CPTD_t));

    cpt.tone = nTone;
    cpt.signal = IFX_TAPI_TONE_CPTD_DIRECTION_TX;
    ret = ioctl( nDataCh_FD, IFX_TAPI_TONE_CPTD_START, (IFX_int32_t) &cpt );
    if (IFX_ERROR == ret)
    {
        TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
             ("Error start CPT detector using fd %d. \
             (File: %s, line: %d)\n", 
             (int) nDataCh_FD, __FILE__, __LINE__));
        return IFX_ERROR;
    }

    return IFX_SUCCESS;
}

/**
   Stop Call Progress Tone Detection
   
   \param nDataCh_FD - data channel file descriptor   
   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t FXO_Stop_CPTD(IFX_int32_t nDataCh_FD)
{
    IFX_return_t ret = IFX_SUCCESS;

   /* check input parameters */
   TD_PARAMETER_CHECK((0 >= nDataCh_FD), nDataCh_FD, IFX_ERROR);    

   IFX_TAPI_TONE_CPTD_t cpt;

   memset(&cpt, 0, sizeof(IFX_TAPI_TONE_CPTD_t));

   cpt.signal = IFX_TAPI_TONE_CPTD_DIRECTION_TX;
   ret = ioctl( nDataCh_FD, IFX_TAPI_TONE_CPTD_STOP, (IFX_int32_t) 0 );
   if ( IFX_ERROR == ret )
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
            ("Error stop CPT detector using fd %d."
             "(File: %s, line: %d)\n", 
             nDataCh_FD, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Initialize FXO channels

   \param pBoard - handle to board
   \param eType - devices to which access is done by GPIO
 * \param eFxoType - FXO device type (Teridian or DuSLIC)

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t FXO_Init(BOARD_t* pBoard, GPIO_ACCESS_DEV_t eType, 
                      FXO_TYPE_t eFxoType)
{
   IFX_uint32_t ret = IFX_SUCCESS;

   /* check input parameters */
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   switch (eFxoType) 
   {
#ifdef DUSLIC_FXO
      case FXO_TYPE_DUSLIC:
         ret = DUSLIC_Init(pBoard, eType);
      break;
#endif
#ifdef TERIDIAN_FXO
      case FXO_TYPE_TERIDIAN:
         ret = TERIDIAN_Init(pBoard, eType);
      break;
#endif
      default:
         /* Unknown FXO type */
         ret = IFX_ERROR;
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Unknown FXO type (%d)."
            "(File: %s, line: %d)\n", 
            (int) eFxoType, __FILE__, __LINE__));
      break;

   } /* switch */

   return ret;
}

/**
   Set new timeout time.
   
   \param pFXO - pointer to fxo
   
   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t FXO_RefreshTimeout(FXO_t* pFXO)
{
   /* check input parameters */
   TD_PTR_CHECK(pFXO, IFX_ERROR);

   if (TD_TIMEOUT_GET_TIME(&pFXO->oRingEventTimeout) != 0)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, FXO No %d: %s (ch %d): Error getting time - gettimeofday(). "
             "(File: %s, line: %d)\n",
             pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             pFXO->nFxoCh, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FXO No %d: %s (ch %d): "
          "Ringing stop occured at %d sec, %d usec.\n",
          pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, pFXO->nFxoCh,
          (IFX_int32_t) pFXO->oRingEventTimeout.tv_sec,
          (IFX_int32_t) pFXO->oRingEventTimeout.tv_usec));

   return IFX_SUCCESS;
}

/**
   End FXO connection (stop pcm connection, unmap mapped channels)
   
   \param pProgramArg - program arguments
   \param pFXO - pointer to fxo
   \param pPhone - pointer to phone
   \param pConn - used connection for fxo 
   \param pBoard - pointer of board
   
   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t FXO_EndConnection(PROGRAM_ARG_t* pProgramArg,
                               FXO_t* pFXO,
                               PHONE_t* pPhone,
                               CONNECTION_t* pConn,
                               BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   CONNECTION_t* pUsedConn = IFX_NULL;
   TAPIDEMO_PORT_DATA_t oPortData;

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);
   TD_PTR_CHECK(pFXO, IFX_ERROR);
   TD_PTR_CHECK(pPhone, IFX_ERROR);
   TD_PTR_CHECK(pConn, IFX_ERROR);
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   /* check if proper FXO state */
   if (S_READY == pFXO->nState)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d invalid  state %s (File: %s, line: %d)\n",
             pFXO->nFXO_Number,
             Common_Enum2Name(pFXO->nState, TD_rgStateName),
             __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* check if ringing on FXO channel detected */
   if (S_IN_RINGING == pFXO->nState)
   {
      /* connection will be activated, no need for timeout check */
      TD_TIMEOUT_STOP(pBoard->pCtrl);
   }

   /* check if cid enabled and in ringing state */
   if (pBoard->pCtrl->pProgramArg->oArgFlags.nCID)
   {
      /* check if CID receiving is still on */
      if (IFX_TRUE == pFXO->fCID_FSK_Started)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("FXO No %d: %s (ch %d): Stop CID RX FSK.\n",
                pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, pFXO->nFxoCh));
         /* Stop FSK and change fxo CID flag state */
         CID_StopFSK(VOIP_GetFD_OfCh(pFXO->nDataCh, pBoard));
         pFXO->fCID_FSK_Started = IFX_FALSE;
      } /* if (IFX_TRUE == pFXO->fCID_FSK_Started) */
   } /* if (pCtrl->pProgramArg->oArgFlags.nCID) */

   /* set structure */
   oPortData.nType = PORT_FXO;
   oPortData.uPort.pFXO = pFXO;

   /* check phone state, only active FXO went off-hook and started CPDT */
   if (S_ACTIVE == pFXO->nState)
   {
      /* Selected FXO goes to S_READY and we also send hook event back. */
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
            ("FXO No %d: %s (ch %d): Set ONHOOK.\n", 
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh));
   
      ret = ioctl(pFXO->nFxoCh_FD, IFX_TAPI_FXO_HOOK_SET, IFX_TAPI_FXO_HOOK_ONHOOK);
      if (IFX_ERROR == ret)
      {
          TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                ("FXO No %d: %s (ch %d): ERR on ioctl cmd IFX_TAPI_FXO_HOOK_SET. "
                 "(File: %s, line: %d)\n",
                 (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
                 (int) pFXO->nFxoCh,
                 __FILE__, __LINE__));        
      }

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:FXO_HOOK_ON::%d",(int) pFXO->nFxoCh));
#endif /* ABACUS_TEST */

      ret = FXO_Stop_CPTD(VOIP_GetFD_OfCh(pFXO->nDataCh, pBoard));
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("FXO No %d: %s (ch %d): "
                "Err, Could not end FXO_Start_CPTD on  data ch %d, %s\n."
                "(File: %s, line: %d)\n", 
                (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
                (int) pFXO->nFxoCh,
                pFXO->nDataCh, pBoard->pszBoardName, __FILE__, __LINE__));
      }
#ifdef ABACUS_TEST
      TAPIDEMO_MOD_FXO(("FXO_TEST:CPTD_BUSY_TONE:STOP:%d",(int) pFXO->nDataCh));
      TAPIDEMO_MOD_FXO(("FXO_TEST:DATA_CHANNEL:RELEASE:%d",(int) pFXO->nDataCh));
#endif /* ABACUS_TEST */
   } /* if (S_ACTIVE == pFXO->nState) */

   /* Unmap and release used data channel */
   ret = FXO_ReleaseDataCh(pFXO, pBoard);
   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d: %s (ch %d): "
             "Err, Could not unmap data channel %d to pcm channel %d."
             "(File: %s, line: %d)\n", 
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh, pFXO->nDataCh, pConn->nUsedCh,
             __FILE__, __LINE__));
   }

   /* Check direction of FXO call */
   if ( IFX_TRUE == pFXO->fIncomingFXO )
   {
      /* Note: Incomming call - connection structure is used from FXO 
               Outcomming call - connection structure is used from PHONE */
      pUsedConn = &pFXO->oConn;
   }
   else
   {
      pUsedConn = pConn;
   }

   /* check if teridian */
   if (IFX_NULL != Common_GetDevice_FXO(pFXO->pBoard->pDevice,
                                         FXO_TYPE_TERIDIAN))
   {
      /* disable lec on call end */
      if (IFX_ERROR == Common_LEC_On_PCM_Ch(pUsedConn->nUsedCh, IFX_DISABLE,
                                            pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("PCM ch No %d: %s Enable LEC failed. (File: %s, line: %d)\n", 
                pUsedConn->nUsedCh, pBoard->pszBoardName, __FILE__, __LINE__));    
      }
   }

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("FXO No %d: %s (ch %d): Deactivate PCM on ch %d, %s\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh, (int) pFXO->nFxoCh,
          pFXO->pBoard->pszBoardName));
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("FXO No %d: %s (ch %d): Deactivate PCM on pcm ch %d %s\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (IFX_TRUE == pFXO->fIncomingFXO) ?
          (int) pFXO->nFxoCh : (int) pConn->nUsedCh,
          (int) pFXO->oConn.nUsedCh, pBoard->pszBoardName));
   /* end PCM connection and free PCM timeslots */
   ret = PCM_EndConnection(&oPortData, pProgramArg, pBoard, pUsedConn);

   if (IFX_ERROR == ret)
   {
       TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
             ("FXO No %d: %s (ch %d): ERR on PCM_EndConnection. "
              "(File: %s, line: %d)\n",
              (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
              (int) pFXO->nFxoCh, __FILE__, __LINE__));        
   }

   /* Unset PCM activation on DUSLIC or Teridian */
   ret = PCM_SetActivation(&oPortData, pFXO->nFxoCh_FD, IFX_FALSE,
                           /* fxo uses two PCM channels on one board, for
                              second channel timeslots must be swaped */
                           pUsedConn->oPCM.nTimeslot_TX,
                           pUsedConn->oPCM.nTimeslot_RX,
                           IFX_FALSE);
#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_IF_FXO_DEACTIVE:%d:%d", 
                     (int) pUsedConn->oPCM.nTimeslot_RX, (int) pFXO->nFxoCh));
#endif /* ABACUS_TEST */

   if (IFX_ERROR == ret)
   {
       TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
             ("FXO No %d: %s (ch %d): ERR on PCM_EndConnection. "
              "(File: %s, line: %d)\n", 
              (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
              (int) pFXO->nFxoCh, __FILE__, __LINE__));
   } 

   if (pFXO->nState == S_ACTIVE)
   {
      /* UnMap pcm channel with phone channel */
      ret = PCM_MapToPhone(pUsedConn->nUsedCh, pPhone->nPhoneCh,
                           IFX_FALSE, pBoard);

#ifdef ABACUS_TEST
      TAPIDEMO_MOD_FXO(("FXO_TEST:PCM2PHONE_PCM:UNMAP:%d",
                        (int) pUsedConn->nUsedCh));
      TAPIDEMO_MOD_FXO(("FXO_TEST:PCM2PHONE_PHONE:UNMAP:%d",
                        (int) pPhone->nPhoneCh));
#endif /* ABACUS_TEST */
   }

   if ( pFXO->fIncomingFXO == IFX_TRUE )
   {
      /* Free the pcm channel in case of incoming call. If there is
         the outcoming call the pcm channel should NOT be free
         becasue it is assigned to the phone channel. */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("FXO No %d: %s (ch %d): Free pcm ch %d, %s\n",
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh,
             (int) pFXO->oConn.nUsedCh, pBoard->pszBoardName));
      PCM_FreeCh(pFXO->oConn.nUsedCh, pBoard);
#ifdef ABACUS_TEST
      TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_CHANNEL:RELEASE:%d",
                        (int) pFXO->oConn.nUsedCh));
#endif /* ABACUS_TEST */
   }

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FXO No %d: %s (ch %d): State changed to U_READY \n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh));
   pFXO->nState = S_READY;

   pPhone->fFXO_Call = IFX_FALSE;
   pPhone->rgoConn[0].nType = UNKNOWN_CALL_TYPE;
   pUsedConn->oPCM.nTimeslot_RX = TIMESLOT_NOT_SET;
   pUsedConn->oPCM.nTimeslot_TX = TIMESLOT_NOT_SET;
   pFXO->pPhone = IFX_NULL;
   pFXO->fFirstStopRingEventOccured = IFX_FALSE;
   /* reset direction marker on end of connection */
   pFXO->fIncomingFXO = IFX_FALSE;

   return ret;
}

/**
   Start FXO connection (start pcm connection, map channels)
   
   \param pProgramArg - program arguments
   \param pFXO - pointer to fxo
   \param pPhone - pointer to phone
   \param pConn - used connection for fxo 
   \param pBoard - pointer of board
   
   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t FXO_StartConnection(PROGRAM_ARG_t* pProgramArg,
                                 FXO_t* pFXO,
                                 PHONE_t* pPhone,
                                 CONNECTION_t* pConn,
                                 BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   IFX_int32_t nTone = BUSY_TONE_IDX;
   TAPIDEMO_PORT_DATA_t oPortData;
   
   /* All the other phones went to S_READY, selected FXO
      goes to S_ACTIVE and we also send hook event back. */

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);
   TD_PTR_CHECK(pFXO, IFX_ERROR);
   TD_PTR_CHECK(pPhone, IFX_ERROR);
   TD_PTR_CHECK(pConn, IFX_ERROR);
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   /* check if proper FXO state */
   if (S_READY != pFXO->nState)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d invalid  state %s, should be state %s"
             "(File: %s, line: %d)\n",
             pFXO->nFXO_Number,
             Common_Enum2Name(pFXO->nState, TD_rgStateName),
             Common_Enum2Name(S_READY, TD_rgStateName),
             __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* set structure */
   oPortData.nType = PORT_FXO;
   oPortData.uPort.pFXO = pFXO;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("FXO No %d: %s (ch %d): "
          "Set OFFHOOK.\n", 
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh));

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:FXO_CHANNEL:SET:%d",(int) pFXO->nFxoCh));
#endif /* ABACUS_TEST */

   ret = ioctl(pFXO->nFxoCh_FD, IFX_TAPI_FXO_HOOK_SET,
               IFX_TAPI_FXO_HOOK_OFFHOOK);
   if (IFX_ERROR == ret)
   {
       TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d: %s (ch %d): "
             "ERR on ioctl cmd IFX_TAPI_FXO_HOOK_SET. "
             "(File: %s, line: %d)\n", 
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh, __FILE__, __LINE__));
   }

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:FXO_HOOK_OFF::%d",(int) pFXO->nFxoCh));
#endif /* ABACUS_TEST */
   
   pFXO->nDataCh = VOIP_GetFreeDataCh(pBoard);
   if (pFXO->nDataCh == NO_FREE_DATA_CH)
   {
      /* ERR could not get free data ch */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("FXO No %d: %s (ch %d): "
            "No data channel available on %s."
            "(File: %s, line: %d)\n", 
            (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
            (int) pFXO->nFxoCh, pBoard->pszBoardName, __FILE__, __LINE__));
   }
   if (VOIP_ReserveDataCh(pFXO->nDataCh, pBoard) != IFX_SUCCESS)
   {
      /* ERR could not reserve data ch */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("FXO No %d: %s (ch %d): "
            "Err, Could not reserve data ch %d, %s."
            "(File: %s, line: %d)\n", 
            (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
            (int) pFXO->nFxoCh,
            pFXO->nDataCh, pBoard->pszBoardName, __FILE__, __LINE__));
   }

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:DATA_CHANNEL:SET:%d",(int) pFXO->nDataCh));
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_CHANNEL:SET:%d",(int) pConn->nUsedCh));
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_RELEASE:NO:%d",(int) pConn->nUsedCh));
#endif /* ABACUS_TEST */

   /* Map pcm channel with data channel */
   ret = PCM_MapToData(pConn->nUsedCh, pFXO->nDataCh,
                       IFX_TRUE, pBoard);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("FXO No %d: %s (ch %d): "
            "Err, Could not map data channel %d to pcm channel %d, %s."
            "(File: %s, line: %d)\n", 
            (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
            (int) pFXO->nFxoCh,
            pFXO->nDataCh, pConn->nUsedCh, pBoard->pszBoardName,
             __FILE__, __LINE__));
   }

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:DATA2PCM_DATA:MAP:%d",(int) pFXO->nDataCh));
   TAPIDEMO_MOD_FXO(("FXO_TEST:DATA2PCM_PCM:MAP:%d",(int) pConn->nUsedCh));

   if (IFX_TRUE == pBoard->pCtrl->nUseCustomCpt)
   {
      /* set CPT index*/
      nTone = COM_GetCustomCPT(nTone);
   }
#endif /* ABACUS_TEST */

   /* Start Call Progress Tone Detection for BUSY_TONE */
   ret = FXO_Start_CPTD(VOIP_GetFD_OfCh(pFXO->nDataCh, pBoard), nTone);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("FXO No %d: %s (ch %d): "
            "Could not start FXO_Start_CPTD on  data ch %d, %s\n."
            "(File: %s, line: %d)\n", 
            (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
            (int) pFXO->nFxoCh,
            pFXO->nDataCh, pBoard->pszBoardName, __FILE__, __LINE__));

      return IFX_ERROR;
   }

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:CPTD_BUSY_TONE:START:%d",(int) pFXO->nDataCh));
#endif /* ABACUS_TEST */

   /* Get timeslots */
   pConn->oPCM.nTimeslot_RX = PCM_GetTimeslots(TS_RX, IFX_FALSE);
   pConn->oPCM.nTimeslot_TX = pConn->oPCM.nTimeslot_RX;

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_IF_ID:SET:%d",(int) pConn->oPCM.nTimeslot_RX));
#endif /* ABACUS_TEST */
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("FXO No %d: %s (ch %d): "
          "Activate PCM on ch %d, %s\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh,
          (int) pFXO->nFxoCh, pFXO->pBoard->pszBoardName));

   /* Set PCM activation on DUSLIC or Teridian*/
   ret = PCM_SetActivation(&oPortData, pFXO->nFxoCh_FD, IFX_TRUE,
                           /* fxo uses two PCM channels on one board, for
                              second (this) channel timeslots must be swaped */
                           pConn->oPCM.nTimeslot_TX,
                           pConn->oPCM.nTimeslot_RX,
                           IFX_FALSE);
   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d: %s (ch %d): ERR on PCM_SetActivation(). "
             "(File: %s, line: %d)\n", 
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh, __FILE__, __LINE__));
   }

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_IF_FXO_ACTIVE:%d:%d",
                     (int) pConn->oPCM.nTimeslot_RX, pFXO->nFxoCh));
#endif /* ABACUS_TEST */

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("FXO No %d: %s (ch %d): "
          "Activate PCM on pcm ch %d, %s\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh,
          (int) pConn->nUsedCh, pBoard->pszBoardName));

   ret = PCM_StartConnection(&oPortData, pProgramArg, pBoard, pConn);
   if (IFX_ERROR == ret)
   {
       TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                        ("FXO No %d: %s (ch %d): "
                         "ERR on PCM_StartConnection. "
                         "(File: %s, line: %d)\n", 
                         (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
                         (int) pFXO->nFxoCh, __FILE__, __LINE__));
   }
   /* check if teridian */
   if (IFX_NULL != Common_GetDevice_FXO(pFXO->pBoard->pDevice,
                                         FXO_TYPE_TERIDIAN))
   {
      /* enable LEC on PCM channel */
      if (IFX_ERROR == Common_LEC_On_PCM_Ch(pConn->nUsedCh, IFX_ENABLE, pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("PCM ch No %d: %s Enable LEC failed. (File: %s, line: %d)\n", 
                pConn->nUsedCh, pBoard->pszBoardName, __FILE__, __LINE__));    
      }
   }
#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PHONE_CHANNEL:SET:%d",(int) pPhone->nPhoneCh));
#endif /* ABACUS_TEST */

   /* Map pcm channel with phone channel of initiator */
   ret = PCM_MapToPhone(pConn->nUsedCh, pPhone->nPhoneCh,
                         IFX_TRUE, pBoard);


#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM2PHONE_PCM:MAP:%d",(int) pConn->nUsedCh));
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM2PHONE_PHONE:MAP:%d",(int) pPhone->nPhoneCh));
#endif /* ABACUS_TEST */

   /* Tell, that we are making FXO connection */
   pPhone->fFXO_Call = IFX_TRUE;

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FXO No %d: %s (ch %d): State changed to S_ACTIVE\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh));
   /* State FXO went S_ACTIVE. */
   pFXO->nState = S_ACTIVE;

   /* reset direction marker on end of connection */
   pFXO->fIncomingFXO = IFX_FALSE;
                 
   pFXO->nPCM_Ch = pConn->nUsedCh;
                 
   return ret;
}


/**
   Prepare FXO connection (start pcm connection between FXO and pcm ch,
                           map data channel)
   
   \param pProgramArg - program arguments
   \param pFXO - pointer to fxo
   \param pBoard - pointer of board
   
   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t FXO_PrepareConnection(PROGRAM_ARG_t* pProgramArg,
                                   FXO_t* pFXO,
                                   BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   IFX_boolean_t action_ok = IFX_FALSE;
   IFX_int32_t nTone = BUSY_TONE_IDX;
   TAPIDEMO_PORT_DATA_t oPortData;

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);
   TD_PTR_CHECK(pFXO, IFX_ERROR);
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   /* check if proper FXO state */
   if (S_READY != pFXO->nState)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d invalid  state %s, should be state %s"
             "(File: %s, line: %d)\n",
             pFXO->nFXO_Number,
             Common_Enum2Name(pFXO->nState, TD_rgStateName),
             Common_Enum2Name(S_READY, TD_rgStateName),
             __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* set structure */
   oPortData.nType = PORT_FXO;
   oPortData.uPort.pFXO = pFXO;

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:FXO_CHANNEL:SET:%d",(int) pFXO->nFxoCh));
#endif /* ABACUS_TEST */

   pFXO->oConn.nType = FXO_CALL;
   /* set FXO direction marker */
   pFXO->fIncomingFXO = IFX_TRUE;
   /* Timeslots are not set */
   pFXO->oConn.oPCM.nTimeslot_RX = TIMESLOT_NOT_SET;
   pFXO->oConn.oPCM.nTimeslot_TX = TIMESLOT_NOT_SET;

   pFXO->nDataCh = VOIP_GetFreeDataCh(pBoard);
   if (pFXO->nDataCh == NO_FREE_DATA_CH)
   {
       /* ERR could not get free data ch */
       TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
             ("FXO No %d: %s (ch %d): "
              "No data channel available on %s."
              "(File: %s, line: %d)\n", 
              (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
              (int) pFXO->nFxoCh, pBoard->pszBoardName, __FILE__, __LINE__));
       return IFX_ERROR;
   }

   if (VOIP_ReserveDataCh(pFXO->nDataCh, pBoard) != IFX_SUCCESS)
   {
       /* ERR could not reserve data ch */
       TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
             ("FXO No %d: %s (ch %d): "
              "Could not reserve data channel %d, %s."
              "(File: %s, line: %d)\n",
              (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
              (int) pFXO->nFxoCh,
              pFXO->nDataCh, pBoard->pszBoardName, __FILE__, __LINE__));
       return IFX_ERROR;
    }

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:DATA_CHANNEL:SET:%d",(int) pFXO->nDataCh));
#endif /* ABACUS_TEST */

   /* Get free pcm channel */
   pFXO->nPCM_Ch = PCM_GetFreeCh(pBoard);
   if (pFXO->nPCM_Ch == NO_FREE_PCM_CH)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d: %s (ch %d): "
             "Err, No free PCM channel on %s. "
             "File: %s, line: %d)\n",
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh, pBoard->pszBoardName,
             __FILE__, __LINE__));
      VOIP_FreeDataCh(pFXO->nDataCh, pBoard);
      return IFX_ERROR;
   }

   if (PCM_ReserveCh(pFXO->nPCM_Ch, pBoard) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d: %s (ch %d): "
             "Err, Could not reserve PCM ch %d, %s. \
             File: %s, line: %d)\n",
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh, 
             pFXO->nPCM_Ch, pBoard->pszBoardName, __FILE__, __LINE__));
      VOIP_FreeDataCh(pFXO->nDataCh, pBoard);
      return IFX_ERROR;
   }

   pFXO->oConn.nUsedCh = pFXO->nPCM_Ch;

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_CHANNEL:SET:%d",(int) pFXO->oConn.nUsedCh));
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_RELEASE:YES:%d",(int) pFXO->oConn.nUsedCh));
#endif /* ABACUS_TEST */

   /* Map pcm channel with data channel */
   ret = PCM_MapToData(pFXO->oConn.nUsedCh, pFXO->nDataCh,
                       IFX_TRUE, pBoard);

   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d: %s (ch %d): "
             "Could not map data channel %d to pcm channel %d, %s."
             "(File: %s, line: %d)\n", 
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh,
             pFXO->nDataCh, pFXO->oConn.nUsedCh, pBoard->pszBoardName,
             __FILE__, __LINE__));
      PCM_FreeCh(pFXO->oConn.nUsedCh, pBoard);
      VOIP_FreeDataCh(pFXO->nDataCh, pBoard);
      return IFX_ERROR;
   }

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:DATA2PCM_DATA:MAP:%d",(int) pFXO->nDataCh));
   TAPIDEMO_MOD_FXO(("FXO_TEST:DATA2PCM_PCM:MAP:%d",(int) pFXO->oConn.nUsedCh));

   if (IFX_TRUE == pBoard->pCtrl->nUseCustomCpt)
   {
      /* set CPT index*/
      nTone = COM_GetCustomCPT(nTone);
   }
#endif /* ABACUS_TEST */

   /* Start Call Progress Tone Detection for BUSY_TONE */
   ret = FXO_Start_CPTD(VOIP_GetFD_OfCh(pFXO->nDataCh, pBoard), nTone);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d: %s (ch %d): "
             "Err, Could not start FXO_Start_CPTD on  data ch %d, %s\n."
             "(File: %s, line: %d)\n", 
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
             (int) pFXO->nFxoCh,
             pFXO->nDataCh, pBoard->pszBoardName, __FILE__, __LINE__));
      FXO_ReleaseDataCh(pFXO, pBoard);
      PCM_FreeCh(pFXO->oConn.nUsedCh, pBoard);
      return IFX_ERROR;
   }


#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:CPTD_BUSY_TONE:START:%d",(int) pFXO->nDataCh));
#endif /* ABACUS_TEST */

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FXO No %d: %s (ch %d): "
          "Activate PCM on ch %d, %s\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh,
          (int) pFXO->nFxoCh, pFXO->pBoard->pszBoardName));

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FXO No %d: %s (ch %d): "
          "Activate PCM on pcm ch %d, %s\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh,
          (int) pFXO->nPCM_Ch, pBoard->pszBoardName));

   /* Get timeslots */
   pFXO->oConn.oPCM.nTimeslot_RX = PCM_GetTimeslots(TS_RX, IFX_FALSE);
   pFXO->oConn.oPCM.nTimeslot_TX = pFXO->oConn.oPCM.nTimeslot_RX;
#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_IF_ID:SET:%d",
                     (int) pFXO->oConn.oPCM.nTimeslot_RX));
#endif /* ABACUS_TEST */
   /* Set PCM activation on FXO */
   action_ok = PCM_SetActivation(&oPortData, pFXO->nFxoCh_FD, IFX_TRUE,
                                 /* fxo uses two PCM channels on one board, for
                                    second channel timeslots must be swaped */
                                 pFXO->oConn.oPCM.nTimeslot_TX,
                                 pFXO->oConn.oPCM.nTimeslot_RX,
                                 IFX_FALSE);

   if (action_ok != IFX_TRUE)
   {
      /* error, could not activate pcm */
      return IFX_ERROR;
   }
   /* set PCM active flag */
   pFXO->oConn.fPCM_Active = IFX_TRUE;

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM_IF_FXO_ACTIVE:%d:%d",
                    (int) pFXO->oConn.oPCM.nTimeslot_RX, pFXO->nFxoCh));
#endif /* ABACUS_TEST */
                           
   if (PCM_StartConnection(&oPortData, pProgramArg, pBoard, &pFXO->oConn) != IFX_TRUE)
   {
      ret = IFX_ERROR;
   }
   /* check if teridian */
   if (IFX_NULL != Common_GetDevice_FXO(pFXO->pBoard->pDevice,
                                         FXO_TYPE_TERIDIAN))
   {
      /* enable LEC on PCM channel */
      if (IFX_ERROR == Common_LEC_On_PCM_Ch(pFXO->oConn.nUsedCh,
                                            IFX_ENABLE, pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("PCM ch No %d: %s Enable LEC failed. (File: %s, line: %d)\n",
                pFXO->oConn.nUsedCh, pBoard->pszBoardName, __FILE__, __LINE__));
      }
   }

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FXO No %d: %s (ch %d): State changed to S_IN_RINGING\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh));
   pFXO->nState = S_IN_RINGING;

   /* use timeout to check if ringing timeout occured */
   TD_TIMEOUT_START(pBoard->pCtrl);

   return ret;
}

/**
   Activate FXO connection (map phone channel to it)
   
   \param pFXO - pointer to fxo
   \param pPhone - pointer to phone
   \param pConn - used connection for fxo 
   \param pBoard - pointer of board
   
   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t FXO_ActivateConnection(FXO_t* pFXO,
                                    PHONE_t* pPhone,
                                    CONNECTION_t* pConn,
                                    BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;

   /* check input parameters */
   TD_PTR_CHECK(pFXO, IFX_ERROR);
   TD_PTR_CHECK(pPhone, IFX_ERROR);
   TD_PTR_CHECK(pConn, IFX_ERROR);
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   /* check if proper FXO state */
   if (S_IN_RINGING != pFXO->nState)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d invalid  state %s, should be state %s"
             "(File: %s, line: %d)\n",
             pFXO->nFXO_Number,
             Common_Enum2Name(pFXO->nState, TD_rgStateName),
             Common_Enum2Name(S_IN_RINGING, TD_rgStateName),
             __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* timeout for FXO cahnnel no longer needed */
   TD_TIMEOUT_STOP(pBoard->pCtrl);

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FXO No %d: %s (ch %d): "
          "Set OFFHOOK.\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName,
          (int) pFXO->nFxoCh));

   ret = ioctl(pFXO->nFxoCh_FD,
               IFX_TAPI_FXO_HOOK_SET, IFX_TAPI_FXO_HOOK_OFFHOOK);
   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("FXO No %d: %s (ch %d): "
             "ERR on ioctl cmd IFX_TAPI_FXO_HOOK_SET. "
             "(File: %s, line: %d)\n",
             (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName,
             (int) pFXO->nFxoCh, __FILE__, __LINE__));
      return IFX_ERROR;
   }
#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:FXO_HOOK_OFF::%d",(int) pFXO->nFxoCh));
   TAPIDEMO_MOD_FXO(("FXO_TEST:PHONE_CHANNEL:SET:%d",(int) pPhone->nPhoneCh));
#endif /* ABACUS_TEST */

   /* Map pcm channel with phone channel of initiator */
   ret = PCM_MapToPhone(pConn->nUsedCh, pPhone->nPhoneCh,
                        IFX_TRUE, pBoard);

#ifdef ABACUS_TEST
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM2PHONE_PCM:MAP:%d",(int) pConn->nUsedCh));
   TAPIDEMO_MOD_FXO(("FXO_TEST:PCM2PHONE_PHONE:MAP:%d",(int) pPhone->nPhoneCh));
#endif /* ABACUS_TEST */

   /* Tell, that we are making FXO connection */
   pPhone->fFXO_Call = IFX_TRUE;

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FXO No %d: %s (ch %d): State changed to S_ACTIVE\n",
          (int) pFXO->nFXO_Number, pFXO->pBoard->pszBoardName, 
          (int) pFXO->nFxoCh));
   pFXO->nState = S_ACTIVE;

   return ret;
}

#endif /* FXO */
/** ---------------------------------------------------------------------- */



