

/******************************************************************************

                               Copyright (c) 2008
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file device_vxt.c
   \date 2009-03-19
   \brief Interface implementation for vxt device.

   Application needs it to initialize device.
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "device_vxt.h"

/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/* ============================= */
/* Global variable definition    */
/* ============================= */

#ifdef USE_FILESYSTEM
/** File holding coefficients. */
IFX_char_t* sBBD_CRAM_File_VXT = "bbd_vxt.bin";
#endif /* USE_FILESYSTEM */

/** Device names */
IFX_char_t* sCTRL_DEV_NAME_VXT = "/dev/vxt";

/** Holding device dependent data */
VXT_IO_Init_t VxtDevInitStruct = {0};

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t DEVICE_VXT_Fillin_FW_Ptr(IFX_void_t* pDrvInit, 
                                       TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                       IFX_char_t* pPath);
IFX_void_t DEVICE_VXT_Release_FW_Ptr(IFX_void_t* pDrvInit);
IFX_return_t DEVICE_VXT_GetVersions(BOARD_t* pBoard);
IFX_return_t DEVICE_VXT_Init(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_void_t DEVICE_VXT_DecodeErrnoForDrv(IFX_int32_t nCode);
IFX_return_t DEVICE_VXT_setupChannel(BOARD_t* pBoard, IFX_char_t* pPath);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Fills in firmware pointers.

   \param pDrvInit - handle to VXT_IO_Init_t structure, should not be null
   \param pCpuDevice - pointer to CPU device
   \param pPath - path to FW files for download

   \return IFX_SUCCES on ok, otherwise IFX_ERROR

   \remarks
*/
IFX_return_t DEVICE_VXT_Fillin_FW_Ptr(IFX_void_t* pDrvInit, 
                                      TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                      IFX_char_t* pPath)
{
#ifdef USE_FILESYSTEM
   VXT_IO_Init_t* vinit = (VXT_IO_Init_t *) pDrvInit;
   IFX_int32_t ret_bbd = IFX_ERROR;
   static IFX_uint8_t *p_bbd = IFX_NULL;
   IFX_boolean_t bFoundBinFile = IFX_FALSE;
   IFX_char_t pszFullFilename[MAX_PATH_LEN] = {0};

   /* check input arguments */
   if ((IFX_NULL == vinit) || (pCpuDevice == IFX_NULL) || (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (vinit->pBBDbuf != IFX_NULL)
   {
      /* fw and bbd files already downloaded. If there is need to 
         download new fw and bbd files, first current buffers
         must be released - DEVICE_VXT_Release_FW_Ptr()*/
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
           ("FW and BBD files already downloaded.\n"));
      return IFX_SUCCESS;
   }

   /* Find BBD_CRAM file with default name */
   ret_bbd = Common_CreateFullFilename(pCpuDevice->pszBBD_CRAM_File, 
                                   pPath, pszFullFilename);
   if (IFX_SUCCESS == ret_bbd)
   {
      /* fill in DRAM and PRAM buffers from binary */
         ret_bbd = Common_ReadBinFile(pszFullFilename,
                                      &p_bbd, (IFX_int32_t *) &vinit->bbd_size);
      /* release bbd memory if applicable */
      if (IFX_ERROR == ret_bbd)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error reading file %s. (File: %s, line: %d)\n",
                pCpuDevice->pszBBD_CRAM_File, __FILE__, __LINE__));
   
         if (p_bbd != IFX_NULL)
         {
            free(p_bbd);
            p_bbd = IFX_NULL;
         }
      }
      else if (p_bbd != IFX_NULL)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, ("Filesystem BBD binary used (%s%s)\n",
                                          pPath, pCpuDevice->pszBBD_CRAM_File));
         vinit->pBBDbuf = p_bbd;
            bFoundBinFile = IFX_TRUE;
      }
   }
   if (!bFoundBinFile)
  {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error, BBD file %s is not found.\n(File: %s, line: %d)\n",
               pCpuDevice->pszBBD_CRAM_File, __FILE__, __LINE__));
      return IFX_ERROR;
  }
#endif /* USE_FILESYSTEM */

  return IFX_SUCCESS;
}


/**
   Release firmware pointers.

   \param pDrvInit - handle to VXT_IO_Init_t structure

   \return none

   \remarks
*/
IFX_void_t DEVICE_VXT_Release_FW_Ptr(IFX_void_t* pDrvInit)
{
#if 1 /* debug RMi */
   VXT_IO_Init_t* vinit = (VXT_IO_Init_t *) pDrvInit;

   /* check input arguments */
   if (vinit == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return;
   }
#ifdef USE_FILESYSTEM
   if (vinit->pBBDbuf != IFX_NULL)
   {
      free(vinit->pBBDbuf);
      vinit->pBBDbuf = IFX_NULL;
   }
#endif /* USE_FILESYSTEM */
#endif
} /* DEVICE_VXT_Release_FW_Ptr() */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Read version after succesfull initialization and display it.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark for this device function not implemented
*/
IFX_return_t DEVICE_VXT_GetVersions(BOARD_t* pBoard)
{
   return IFX_SUCCESS;
} /* DEVICE_VXT_GetVersions() */

/**
   Init device.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download
   
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t DEVICE_VXT_Init(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_TAPI_CH_INIT_t Init;
   IFX_TAPI_LINE_FEED_t lineFeed;
   IFX_TAPI_RING_CADENCE_t ring;
   IFX_int32_t i = 0;
   /* pattern of 3 s ring and 1 s pause */
   char data[10] = { 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xF0,0x00,0x00 };
   /* initial 1 s ring and 1,8 s non ringing signal before ringing */
   char initial[7] ={0xFF, 0xFF, 0xF0, 0x00, 0x00, 0x00, 0x00};
   IFX_return_t ret;
   IFX_TAPI_CALIBRATION_t calibration;
   IFX_TAPI_EVENT_t event;
   /* used to check if previous part of initialization was successfull */
   IFX_int32_t* pChannelTable;
   TAPIDEMO_DEVICE_t      *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t  *pCpuDevice = IFX_NULL;

   /* check input arguments */
   if ((pBoard == IFX_NULL) || (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, CPU device not found. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

   DEVICE_VXT_Fillin_FW_Ptr(pCpuDevice->pDevDependStuff, pCpuDevice, pPath);

   /* Set tapi init structure */
   memset(&Init, 0, sizeof(IFX_TAPI_CH_INIT_t));
   Init.nMode = IFX_TAPI_INIT_MODE_PCM_PHONE;
   Init.pProc = (IFX_void_t*) pCpuDevice->pDevDependStuff;

#ifdef ABACUS_TEST
   if(nVerifySystemInitializtion)
   {
      /* Read version */
      DEVICE_VXT_GetVersions(pBoard);
      COM_VerifySystemInit(pBoard, &Init);
   }
   else
#endif /* ABACUS_TEST */
   {
      /* allocate memmory */
      pChannelTable = (IFX_int32_t*)calloc(pBoard->nMaxAnalogCh,
                                           sizeof(IFX_int32_t));

      memset (&ring, 0, sizeof(IFX_TAPI_RING_CADENCE_t));
      /* set size in bits */
      memcpy (&ring.data[0], data, sizeof (data));
      ring.nr = sizeof(data) * 8;

      /* set the ring sequence */
      memcpy (&ring.initial[0], initial, sizeof (initial));
      ring.initialNr = sizeof(initial) * 8;

      /* Initialize all tapi channels */
      for (i = 0; i <= pBoard->nMaxAnalogCh; i++)
      {
         /* after all channels are initialized */
         if (pBoard->nMaxAnalogCh == i)
         {
            /* Read version */
            DEVICE_VXT_GetVersions(pBoard);
            break;
         }

         /* Initialize all system channels */
         Init.dev = pBoard->rgoPhones[i].nDev;
         Init.ch = pBoard->rgoPhones[i].nPhoneCh;
         TD_ASSERT ((ret = ioctl(pBoard->nDevCtrl_FD, IFX_TAPI_CH_INIT,
            (IFX_int32_t) &Init)) == IFX_SUCCESS, pBoard->nDevCtrl_FD, pBoard,
            Init.dev, Init.ch);
         if (IFX_SUCCESS != ret)
         {
            pChannelTable[i] = IFX_ERROR;
         }
         else
         {
            pChannelTable[i] = IFX_SUCCESS;
         }
      } /* for all tapi channels*/
      /* Start analog line calibration process for all tapi channels */
      for (i = 0; i < pBoard->nMaxAnalogCh; i++)
      {
         /* if previous step of initialiation failed then skip this one */
         if (IFX_ERROR == pChannelTable[i])
         {
            continue;
         }
         memset(&calibration, 0, sizeof(IFX_TAPI_CALIBRATION_t));
         /* Calibrate all system channels */
         calibration.dev = pBoard->rgoPhones[i].nDev;
         calibration.ch = pBoard->rgoPhones[i].nPhoneCh;
         TD_ASSERT ((ret = ioctl(pBoard->nDevCtrl_FD, 
            IFX_TAPI_CALIBRATION_START,
            (IFX_int32_t) &calibration)) == IFX_SUCCESS, pBoard->nDevCtrl_FD,
            pBoard, calibration.dev, calibration.ch);
         if (IFX_SUCCESS != ret)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("IFX_TAPI_CALIBRATION_START failed for device %d channel %d. "
                "(File: %s, line: %d)\n",
                calibration.dev, calibration.ch, __FILE__, __LINE__));
            pChannelTable[i] = IFX_ERROR;;
         }
      } /* for all tapi channels*/
      /* wait for calibration process end */
      VOS_Sleep(1500);
      /* Check if callibration end event was received for all tapi channels */
      for (i = 0; i < pBoard->nMaxAnalogCh; i++)
      {
         /* if previous step of initialiation failed then skip this one */
         if (IFX_ERROR == pChannelTable[i])
         {
            continue;
         }
         /* check if callibration end event was received */
         do
         {         
            memset(&event, 0, sizeof(IFX_TAPI_EVENT_t));
            event.ch = pBoard->rgoPhones[i].nPhoneCh;
            event.dev = pBoard->rgoPhones[i].nDev;
            TD_ASSERT ((ret = ioctl(pBoard->nDevCtrl_FD, IFX_TAPI_EVENT_GET, 
               (IFX_int32_t) &event)) == IFX_SUCCESS, pBoard->nDevCtrl_FD,
               pBoard, event.dev, event.ch);
            if (IFX_TAPI_EVENT_NONE == event.id)
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("Didn't receive calibration end event for "
                      "device %d channel %d. "
                      "(File: %s, line: %d)\n",
                      pBoard->rgoPhones[i].nDev, pBoard->rgoPhones[i].nPhoneCh,
                      __FILE__, __LINE__));
               pChannelTable[i] = IFX_ERROR;
            }
            if ((IFX_TAPI_EVENT_CALIBRATION_END == event.id) &&
                (IFX_TAPI_EVENT_CALIBRATION_SUCCESS != event.data.calibration))
            {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                     ("Receive calibration end event with error for "
                      "device %d channel %d. "
                      "(File: %s, line: %d)\n",
                      event.dev, event.ch, __FILE__, __LINE__));
               pChannelTable[i] = IFX_ERROR;
            }
         }
         while ((IFX_TAPI_EVENT_CALIBRATION_END != event.id) &&
                (IFX_TAPI_EVENT_NONE != event.id));

      } /* for all tapi channels*/
      /* Set appropriate feeding on all (analog) line channels */
      for (i = 0; i < pBoard->nMaxAnalogCh; i++)
      {
         /* if previous step of initialiation failed then skip this one */
         if (IFX_ERROR == pChannelTable[i])
         {
            continue;
         }
         /* Set appropriate feeding on all (analog) line channels */
         /* Set line in standby */
         lineFeed.dev = pBoard->rgoPhones[i].nDev;
         lineFeed.ch = pBoard->rgoPhones[i].nPhoneCh;
         lineFeed.lineMode = IFX_TAPI_LINE_FEED_STANDBY;
         TD_ASSERT ((ret = ioctl(pBoard->nDevCtrl_FD, IFX_TAPI_LINE_FEED_SET,
            (IFX_int32_t) &lineFeed)) == IFX_SUCCESS, pBoard->nDevCtrl_FD,
            pBoard, lineFeed.dev, lineFeed.ch);
         if (IFX_SUCCESS != ret)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("IFX_TAPI_LINE_FEED_STANDBY set failed for "
               "%s dev %d, ch %d, "
               "(File: %s, line: %d)\n",
               pBoard->pszBoardName, lineFeed.dev, lineFeed.ch, 
               __FILE__, __LINE__));
            continue;
         }
         ring.dev = pBoard->rgoPhones[i].nDev;
         ring.ch = pBoard->rgoPhones[i].nPhoneCh;
         TD_ASSERT ((ret = ioctl(pBoard->nDevCtrl_FD,
            IFX_TAPI_RING_CADENCE_HR_SET, (IFX_int32_t) &ring)) == IFX_SUCCESS,
            pBoard->nDevCtrl_FD, pBoard, ring.dev, ring.ch);
         if (ret != IFX_SUCCESS)
         {
            continue;
         }
         VOS_Sleep(100);
      } /* for all tapi channels*/

      /* release allocated memory */
      free(pChannelTable); 
   }

   DEVICE_VXT_Release_FW_Ptr(pCpuDevice->pDevDependStuff);

#ifdef ABACUS_TEST
   if(!nVerifySystemInitializtion)
#endif /* ABACUS_TEST */
   {
      if (pBoard->nMaxAnalogCh != i)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Init failed for dev %d, ch %d, (File: %s, line: %d)\n",
               pBoard->rgoPhones[i].nDev, pBoard->rgoPhones[i].nPhoneCh,
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   return IFX_SUCCESS;
} /* DEVICE_VXT_Init() */

/**
   Handles the DRIVER error codes and stack.

   \param nCode - error code

   \remarks
*/
IFX_void_t DEVICE_VXT_DecodeErrnoForDrv(IFX_int32_t nCode)
{
   IFX_int32_t i = 0;

   for (i = 0; i < VXT_ERRNO_CNT; ++i)
   {
      if (VXT_drvErrnos[i] == (nCode & 0xFFFF))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("LL Err: %s\n", VXT_drvErrStrings[i]));
         break;
      }
   }
} /* DEVICE_VXT_DecodeErrnoForDrv() */

/**
   Return last error number for driver.

   \param nFd  -  channel file descriptor

   \return number of error if successful, otherwise IFX_ERROR.
   \remarks
*/
IFX_return_t DEVICE_VXT_GetErrnoNum(IFX_int32_t nFd)
{
   return IFX_SUCCESS;
}

/**
   Downloads fw and bbd files and calls IFX_TAPI_CH_INIT for first channel.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks It is necessary to call this function to have working the
    IFX_TAPI_CAP_LIST. 
*/
IFX_return_t DEVICE_VXT_setupChannel(BOARD_t* pBoard, IFX_char_t* pPath)
{
   /* not needed for VXT. It is done by other application during 
      board startup.*/
   return IFX_SUCCESS;
}

/**
   Register device to the board.

   \param pCpuDevice - pointer to device
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t DEVICE_VXT_Register(TAPIDEMO_DEVICE_CPU_t* pCpuDevice)
{
   /* check input arguments */
   if (pCpuDevice == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice->Version = DEVICE_VXT_GetVersions;
   pCpuDevice->DecodeErrno = DEVICE_VXT_DecodeErrnoForDrv;
   pCpuDevice->GetErrno = DEVICE_VXT_GetErrnoNum;
#ifdef ABACUS_TEST
   pCpuDevice->FillinFwPtr = DEVICE_VXT_Fillin_FW_Ptr;
   pCpuDevice->ReleaseFwPtr = DEVICE_VXT_Release_FW_Ptr;
#endif /* ABACUS_TEST */
   return IFX_SUCCESS;
}

